@@ -11,6 +11,7 @@ use {
     },
     solana_gossip::{contact_info::ContactInfo, gossip_service::discover},
     solana_sdk::pubkey::Pubkey,
+    solana_streamer::socket::SocketAddrSpace,
     std::{
         error,
         net::{IpAddr, Ipv4Addr, SocketAddr},
@@ -32,6 +33,13 @@ fn parse_matches() -> ArgMatches<'static> {
         .about(crate_description!())
         .version(solana_version::version!())
         .setting(AppSettings::SubcommandRequiredElseHelp)
+        .arg(
+            Arg::with_name("allow_private_addr")
+                .long("allow-private-addr")
+                .takes_value(false)
+                .help("Allow contacting private ip addresses")
+                .hidden(true),
+        )
         .subcommand(
             SubCommand::with_name("rpc-url")
                 .about("Get an RPC URL for the cluster")
@@ -224,6 +232,7 @@ fn process_spy(matches: &ArgMatches) -> std::io::Result<()> {
     let pubkey = matches
         .value_of("node_pubkey")
         .map(|pubkey_str| pubkey_str.parse::<Pubkey>().unwrap());
+    let socket_addr_space = SocketAddrSpace::new(matches.is_present("allow_private_addr"));
     let shred_version = value_t_or_exit!(matches, "shred_version", u16);
     let identity_keypair = keypair_of(matches, "identity").map(Arc::new);
 
@@ -251,6 +260,7 @@ fn process_spy(matches: &ArgMatches) -> std::io::Result<()> {
         None,               // find_node_by_gossip_addr
         Some(&gossip_addr), // my_gossip_addr
         shred_version,
+        socket_addr_space,
     )?;
 
     process_spy_results(timeout, validators, num_nodes, num_nodes_exactly, pubkey);
@@ -273,6 +283,7 @@ fn process_rpc_url(matches: &ArgMatches) -> std::io::Result<()> {
     let entrypoint_addr = parse_entrypoint(matches);
     let timeout = value_t_or_exit!(matches, "timeout", u64);
     let shred_version = value_t_or_exit!(matches, "shred_version", u16);
+    let socket_addr_space = SocketAddrSpace::new(matches.is_present("allow_private_addr"));
     let (_all_peers, validators) = discover(
         None, // keypair
         entrypoint_addr.as_ref(),
@@ -282,13 +293,14 @@ fn process_rpc_url(matches: &ArgMatches) -> std::io::Result<()> {
         entrypoint_addr.as_ref(), // find_node_by_gossip_addr
         None,                     // my_gossip_addr
         shred_version,
+        socket_addr_space,
     )?;
 
     let rpc_addrs: Vec<_> = validators
         .iter()
         .filter_map(|contact_info| {
             if (any || all || Some(contact_info.gossip) == entrypoint_addr)
-                && ContactInfo::is_valid_address(&contact_info.rpc)
+                && ContactInfo::is_valid_address(&contact_info.rpc, &socket_addr_space)
             {
                 return Some(contact_info.rpc);
             }
